/*
 * coins.X - Practicum, Spring 2009
 *
 * Anatole Paine and Emma Carlson
 *
 * May 16, 2009
 */

#include <cstddef>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

using namespace std;

// Constants from the problem
const int MAX_COINS = 1000;
const int MAX_WEIGHINGS = 3000;

bool DEBUG = true;

// A nice pair of data structures to hold the information we need in our
// nodes and edges.
struct Edge
{
    int source;
    int dest;

    // Since std::find needs an equality operator, we'll provide one.
    bool operator==(Edge rhs)
    {
        return source == rhs.source && dest == rhs.dest;
    }
};

struct Node
{
    int label;
    vector<int> in;
    vector<int> out;

    Node(int _label)
        : label(_label), in(), out()
    {
        // Nothing to do here.
    }
};

// A funtion call that should only be run if the graph is discovered to be
// cyclic; Prints "IMPOSSIBLE" and then exits the program.
void impossible()
{
    cout << "IMPOSSIBLE" << endl;
    exit(0);
}

/*
 * Helper function: Computes whether a given node is a "source" - i.e. it has
 * no incoming edges - from a set of edges.
 */
bool source(const vector<Edge>& edges, int node)
{
    for (size_t i = 0; i < edges.size(); ++i)
    {
        if (edges.at(i).dest == node)
            return false;
    }
    return true;
}

/*
 * Function: Topological sort. This is used to make sure that we have no cycles
 * in the graph created by the coins, since a partial ordering is only valid on
 * a DAG.
 */
void topoSort(const vector<Node>& nodes, vector<Edge> edges)
{
    vector<int> sources;

    /* 
     * We need to go through and find which nodes are sources (i.e. they have
     * only outgoing edges), and make sure that these are the only node labels
     * in our sources vector.
     */
    for (size_t i = 0; i < nodes.size(); ++i)
        if (nodes.at(i).in.empty())
        {
            //cerr << "Adding node " << i << " to sources" << endl;
            sources.push_back(nodes.at(i).label);
        }

    /*
     * Now for the algorithm. Essentially, we start by finding the source nodes
     * (nodes without incoming edges), and put them onto a stack. Then, we pop
     * each of them off the stack, remove their outgoing edges, and push any
     * newly-source nodes onto the stack. Once this process ends, if the graph
     * really is a DAG, then we will have removed all of the edges in it.
     */
    while (!sources.empty())
    {
        int current = sources.back();
        sources.pop_back();
        
        vector<Edge> newEdges = edges;
        for (size_t i = 0; i < edges.size(); ++i)
        {
            if (current == edges.at(i).source)
            {
                /* Debug code
                cerr << "sources is currently [";
                for (size_t j = 0; j < sources.size(); ++j)
                {
                    cerr << sources.at(j) + 1 << ", ";
                }
                cerr << "]" << endl;

                cerr << "Currently looking at edge {" << edges.at(i).source + 1;
                cerr << ", " << edges.at(i).dest + 1 << "}" << endl;
                */

                vector<Edge>::iterator found;
                found = find(newEdges.begin(), newEdges.end(), edges.at(i));
                newEdges.erase(found);

                /* More debug code.`
                cerr << "newEdges is now ";
                for (size_t j = 0; j < newEdges.size(); ++j)
                {
                    cerr << "[" << newEdges.at(j).source + 1 << ", ";
                    cerr << newEdges.at(j).dest + 1 << "], ";
                }
                cerr << endl;
                */

                if (source(newEdges, edges.at(i).dest))
                {
                    sources.push_back(edges.at(i).dest);
                    //cerr << "Adding node " << edges.at(i).dest + 1 << " to sources" << endl;
                }
            }
        }
        edges = newEdges;
    }

    // If we were given a cyclic graph, complain loudly.
    if (!edges.empty())
        impossible();
}

/*
 * Helper function: Find the number of coins (nodes) that the given node is
 * distinct from. We do this by finding all of the paths into and out of the
 * given node, keeping track of which nodes we've seen before so as to not
 * double-count.
 */
int findDistinct(const vector<Node>& nodes, const int currentNode)
{
    // Find all paths into the given node.
    vector<bool> marked = vector<bool>(nodes.size(), false);
    vector<int> in;
    in.push_back(currentNode);

    // Setting pathsIn to -1 counteracts the fact that the following while loop
    // will increment it once more than necessary.
    int pathsIn = -1;

    /*
     * The purpose of these two loops is to look at every path in or out of
     * the current node, by following all of the paths from or to it.
     */
    while (!in.empty())
    {
        int current = in.back();
        in.pop_back();
        if (!marked.at(current))
        {
            marked.at(current) = true;
            vector<int> adjacent = nodes.at(current).in;
            
            for (size_t i = 0; i < adjacent.size(); ++i)
                in.push_back(adjacent.at(i));
            
            ++pathsIn;
        }
    }

    // Find all paths out of the given node.
    marked = vector<bool>(nodes.size(), false);
    vector<int> out;
    out.push_back(currentNode);

    // Again, setting pathsOut to -1 counteracts the fact that the following
    // while loop will increment it once more than necessary.
    int pathsOut = -1;

    while (!out.empty())
    {
        int current = out.back();
        out.pop_back();
        if (!marked.at(current))
        {
            marked.at(current) = true;
            vector<int> adjacent = nodes.at(current).out;

            for (size_t i = 0; i < adjacent.size(); ++i)
                out.push_back(adjacent.at(i));
            
            ++pathsOut;
        }
    }

    return pathsIn + pathsOut;
}

/*
 * Primary function: Generates a list of the levels of distinctness of all of
 * the coins, and then returns the first incidence of the maximum level of
 * distinctness.
 */
int mostDistinct(const vector<Node>& nodes)
{
    vector<int> distinct = vector<int>(nodes.size(), 0);
    for(size_t i = 0; i < nodes.size(); ++i)
    {
        Node current = nodes.at(i);
        int currDistinct = findDistinct(nodes, current.label);
        distinct.at(current.label) = currDistinct;
    }

    // Find and return the index of the first coin that is most distinct.
    vector<int>::iterator maxDistinct = max_element(distinct.begin(),
                                                    distinct.end());
    size_t maxIndex = 0;
    for (maxIndex = 0; maxIndex < distinct.size(); ++maxIndex)
        if (distinct.at(maxIndex) == *maxDistinct)
            break;

    return maxIndex + 1;
}

/*
 * The main driver for the program. In this, we handle all of the ugly input
 * processing, and handle any exceptions that come up. Note that in this
 * program, we exclusively use the vector::at() accessor method instead of
 * vector::operator[], as at() is safer - it throws an exception where normal
 * subscripting would simply let you read or write off the end of the vector.
 */
int main() // We don't care about command-line stuff.
{
    
    // Let's get some input.
    try
    {
        int source = 0;
        int dest = 0;
        int numArcs = 0;
        int numCoins = 0;

        cin >> numCoins >> numArcs;
        
        /*
         * We need to make sure that we avoid going outside the limits set by
         * the problem. Note that none of these are program-crashing problems,
         * but we'll quit anyway for safety.
         */
        if (numArcs > MAX_WEIGHINGS)
        {
            cerr << "Too many weighings!" << endl;
            exit(1);
        }
        if (numCoins > MAX_COINS)
        {
            cerr << "Too many coins!" << endl;
            exit(1);
        }

        /* 
         * Now that everything is well and good, we construct our graph.
         * First, we'll create the list of nodes - note that we will be calling
         * the nodes by their indices, so coin 1 becomes node 0, etc. Then, for
         * each line, we will add a new edge to the list of edges, and add the
         * information to the relevant nodes if need be.
         */
        vector<Node> nodes;
        for (size_t i = 0; i < numCoins; ++i)
        {
            Node current = Node(i);
            nodes.push_back(current);
        }

        vector<Edge> edges;
        for (int i = 0; i < numArcs; ++i)
        {
            cin >> source >> dest;
            
            if (source > numArcs || dest > numArcs)
            {
                cerr << "Invalid coin number!" << endl;
                exit(1);
            }

            Edge current = {source - 1, dest - 1};
            edges.push_back(current);

            // Add the information given by the edge to the nodes involved.
            nodes.at(source - 1).out.push_back(dest - 1);
            nodes.at(dest - 1).in.push_back(source - 1);
        }

        /* 
         * Now, we can finally do some computation. First, we topologically
         * sort the graph to make sure it contains no cycles. Then all we need
         * to do is use mostDistinct to find the most distinct coin, and output
         * it.
         */
        topoSort(nodes, edges);
        int bestCoin = mostDistinct(nodes);
        cout << bestCoin << endl;
    }
    catch(const exception& e)
    {
        cerr << "Something went wrong:" << e.what() << endl;
        return(1);
    }

    return 0;
}
