/*
 * mstore.X - Practicum, Spring 2009
 *
 * Anatole Paine and Emma Carlson
 *
 * May 15, 2009
 */

#include <algorithm>
#include <cstddef>
#include <cmath>
#include <cstdlib>
#include <cassert>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <string>
#include <vector>
#include <sstream>

using namespace std;

// Constants from the problem.
const int MAX_C = 1000000;
const int MAX_T = 1440;
const int MAX_P = 1000000;
const int MAX_N = 2000;

// A struct to store customer data in, just to make our lives easier.
struct Customer
{
    int t;
    int p;
};


/*
 * Function: Maximal contiguous section. Given a vector, computes the value
 * of the maximal continuous subsection of the vector, by tallying the total
 * value at each point, keeping track of the maximum reached so far, and
 * resetting the tally to zero if it ever becomes negative.
 */
int mcs(const vector<int>& profits)
{
    int max = 0;
    int currentMax = 0;

    for (int i = 0; i <= MAX_T; ++i)
    {
        currentMax += profits.at(i);
        
        if (currentMax < 0)
            currentMax = 0;

        if (currentMax > max)
            max = currentMax;
    }
    return max;
}

/*
 * Main function: Find the most profit Farmer Ran can make given the customers
 * he has. Essentially, this iterates over the prices each customer is willing
 * to pay, and uses the mcs() helper function to see how much FR can make in one
 * sitting selling milk at that price.
 */
int mostProfit(const vector<Customer>& customers, const int charge)
{ 
    int max = 0;
    
    for (int i = 0; i < customers.size(); ++i)
    {
        /* 
         * Initalize the data structure holding our profits for each minute.
         * We start by initializing all of the elements to be -C, as for times
         * when FR has no customers, he is losing money.
         */
        vector<int> profits(MAX_T + 1, -charge);
        
        int currentPrice = customers.at(i).p;
        for (int j = 0; j < customers.size(); ++j)
        {

            /* 
             * This is important: Each customer will only pay so much for milk,
             * so if FR sets the price higher than what they are willing to pay,
             * we do not include them in the list of profits FR will make.
             */
            Customer cust = customers.at(j);
            if (cust.p >= currentPrice)
                profits.at(cust.t) += currentPrice;
        }

        // Check the profit gained from the current price against previous
        // profits.
        int currentMax = mcs(profits);
        max = currentMax > max ? currentMax : max;
    }
    return max;
}

/*
 * Function: The main driver for the program. This handles all of the ugly
 * input stuff, turns the input into useful data, and then runs the algorithm
 * on that data.
 */
int main() // We don't care about command-line stuff.
{
    try
    {
        // Get N, L, and P.
        string firstLine;
        getline(cin, firstLine);
        stringstream first(firstLine);
        int N = 0;
        int C = 0;
        first >> N >> C;

        /*
         * We need to make sure that we avoid going outside the limits set by
         * the problem. Note that none of these are program-crashing problems,
         * but we'll quit anyway for safety.
         */
        if (N > MAX_N || C > MAX_C)
        {
            cerr << "N or C too large!" << endl;
            exit(1);
        }
        

        // vector of customers in {t, p} pairs
        vector<Customer> customers(N);
        
        /* 
         * For the next N lines, get a pair of numbers, a time and a revenue
         * for that time. Note that while doing this, we subtract the given
         * profit at that minute from the overall cost per minute of operating
         * the store.
         */
        for(int i = 0; i < N; ++i)
        {
            // Initialize T_i and P_i to -1 for error checking purposes.
            int t = -1;
            int p = -1;

            // Get T_i and P_i, pair them, and push them onto our vector
            // of Customers.
            cin >> t >> p;

            if (t > MAX_T)
            {
                cerr << "Time greater than 1440" << endl;
                exit(1);
            }

            if (p > MAX_P)
            {
                cerr << "Revenue at minute " << t << " greater than $1M!" << endl;
                exit(1);
            }

            Customer current = {t, p};
            customers.push_back(current);
        }

        /* 
         * Now, we can finally do some computation. Since mostProfit returns
         * an int, we simply print it.
         */
        cout << mostProfit(customers, C) << endl;
    }
    catch(const exception& e)
    {
        cerr << "Something went wrong:" << e.what() << endl;
        return(1);
    }

    return 0;
}
