/*
  Polish Notation Parser and Evaluator

  Written by Kevin Oelze and Hannah Hoersting

  Comment Annotations added by Kevin Oelze
*/


/*
 
 KEYWORDS:
 
 Recursive Descent Parsing
 Logical expressions (boolean logic)
 
 Exception-handling in Java
 
 */


/*
If you're not familiar with Scanner, it's the best input parser in
Java, and you should get very familiar with it.

Exceptions are much less important, but can be
used for really cool things if you know how.

You'll see more later on, I swear =).
*/

import java.security.AccessControlException;
import java.util.Scanner;


public class logic {


   public static void main (String[] args) {
       Scanner s = new Scanner(System.in);
       while(s.hasNext()) {

           String str = s.nextLine();
           char[] vars = new char[10];
           /* We initialize the matrix containing the list of variables in
              Polish Notation with a special character, in this case '!'

              Since we may have to store up to 10 variables, we use
              an array of size 10 with a sentinel character as those that
              are not actually variables.

           */
           for (int i = 0; i < 10 ; i++) {
               vars[i] = '!';
           }
           try {
               /* We parse our tree as much as possible.
                  More on this later.

               */
               TreeString parseTree = parse(str,vars);

               /* If we finish a valid parse and there's
                  still extra text, there had to be extraneous
                  text, so this must be invalid, since
                  Polish Notation is unambiguous.
               */

               if (!parseTree.str.equals(""))
                   System.out.println(str + " is invalid: extraneous text");
               else {
                   /* At this point, we must have had a valid parse tree,
                      so we must now evaluate if it is a tautology, a
                      contradiction, or contingent on the exact valuation.
                      We'll do this in a pretty cool way.
                   */
                   Tree t= parseTree.tree;
                   /*
                     First we need to figure out exactly how many
                     variables there are.  We can do this by figuring
                     out where the first sentinel character is
                     located.
                   */
                   int num = t.addVar('!');
                   if (num < 0)
                       num = 10;
                   // Variables that tell us whether we've seen a satisfied
                   // or unsatisfied instance
                   boolean yes = false;
                   boolean no = false;
                   /*
                     How we evaluate our assignments is pretty cool here:
                     With n variables, there are 2^n possible valuations,
                     since each variable can only be true or false.  The
                     nifty thing is that this valuation can be represented
                     as a binary string!  We just iterate through all of them
                     here.
                   */
                   for (int i = 0; i < Math.pow(2,num); i++) {
                       String assignments = Integer.toBinaryString(i);
                       // Gotta make sure we have however many leading
                       // zeroes that we need.
                       while (assignments.length() < num) {
                           assignments = "0" + assignments;
                       }
                       //Now we simply evaluate: more on this later, again!
                       boolean worked = eval(t, assignments);
                       //Now we update whether we've found a satisfied or
                       //unsatisfied instance.
                       if (worked)
                           yes = true;
                       else
                           no = true;
                   }

                   /* Now we just use yes and no to tell us what type of
                      valid Polish Notation formula it is, and we're done!
                   */

                   String postfix = "";
                   if (yes && no)
                       postfix = "contingent";
                   else if (!no)
                       postfix = "tautology";
                   else
                       postfix = "contradiction";
                   System.out.println(str + " is valid: " + postfix);
               }
           }
           /*
             Ahh, the beauty of exceptions: the only purpose of these
             exceptions is to break out of the parse tree once I've
             discovered that we have insufficient operands, simply because
             I ran out of things to parse while I was doing a valid parse.

             The =only= way this exception happens is if I throw it.  It's
             a less evil way of using something similar to gotos to escape
             a large recursive chain.

           */
           catch (AccessControlException e) {
               System.out.println(str + " is invalid: insufficient operands");
           }

           /* Similarly so here.  In this case, I throw an
              ArrayIndexOutOfBoundsException if I notice we have an invalid
              character.  Since the entire parsing code is inside a try/catch
              statement, if I throw the exception, I end up out here.
           */

           catch (ArrayIndexOutOfBoundsException e) {
               System.out.println(str + " is invalid: invalid character");
           }
       }
   }

   /* This function is where all of the parsing gets done.  It returns
      a TreeString.  A TreeString is simply a wrapper container that holds
      a Tree and a String.  The String that is returned holds everything that
      is left to be parsed.  Because Polish Notation is LL(0), we can
      greedily parse without ever having to look ahead at the next
      character.  This makes the job of constructing a parse tree fairly
      painless.
    */

   public static TreeString parse(String str, char[] vars) {
       /*
         We've just run out of characters, so let's throw the
         Exception that says there are insufficient operands and
         say hello to Professor Dodds at the same time!
       */
       if (str.equals(""))
           throw new AccessControlException("Hi Prof Dodds!");
       char c = str.charAt(0);
       /*
          We begin constructing our parse tree now.  Obviously, the value at
          the node is going to be the first character in the string we
          parse.  We then parse the rest to see what we want to set the
          left and right subtrees to.  We also pass along our list of
          variables so that each tree knows about the same variables.

       */
       Tree t = new Tree(null, null, c, vars);
       switch(c) {

           /*All of these parse the same, since these all have two valid
             Polish Notation expressions following them.
           */

       case 'C':
       case 'K':
       case 'A':
       case 'D':
       case 'E':
       case 'J':
           //Let the left subtree greedily parse as much as it can.
           TreeString newT = parse(str.substring(1), vars);
           t.left = newT.tree;
           vars = newT.tree.vars;
           //Now let's have the right subtree greedily parse as much as it
           //can.
           TreeString newR = parse(newT.str, vars);
           t.right = newR.tree;
           //Now let's return the parse tree and whatever is left.
           return new TreeString(t, newR.str);
       case 'N':
           //Same theory here as before, but this time we only have one
           //part of the tree to parse.
           TreeString newM = parse(str.substring(1), vars);
           t.left = newM.tree;
           return new TreeString(t,newM.str);
       default:
           if (c < 'a' || c > 'z')
               //Not a valid lowercase variable, so let's mark it as invalid
               //and throw the exception.
               throw new ArrayIndexOutOfBoundsException();
           else {
               //Otherwise we add the variable--we'll see how later.
               int i = t.addVar(c);
               if (i < 0) {
                   //If we can't add the variable, that means we have too
                   //many variables, which means an invalid character in
                   //the scope of this problem.
                   throw new ArrayIndexOutOfBoundsException();
               }
               else
                   //Otherwise we have a completely valid variable.  A
                   //variable is a parse tree that has no children, so
                   //We can just return now.
                   return new TreeString(t, str.substring(1));
           }
       }


   }

   /* This function takes a parse tree and an assignment (as a binary string!)
      we want to give to the variables and decides if that valuation
      satisfies the Polish Notation expression or not.
   */

   public static boolean eval(Tree t, String assignments) {
       char c = t.value;
       /*
          We evaluate the left and right and use the truth table to decide
          what to return now.
       */
       switch(c) {
       case 'C':
           return !(eval(t.left, assignments) && !eval(t.right,assignments));
       case 'N':
           return !(eval(t.left,assignments));
       case 'K':
           return eval(t.left,assignments) && eval(t.right,assignments);
       case 'A':
           return eval(t.left,assignments) || eval(t.right,assignments);
       case 'D':
           return !(eval(t.left,assignments) && eval(t.right,assignments));
       case 'E':
           return eval(t.left,assignments) == eval(t.right,assignments);
       case 'J':
           boolean left = eval(t.left,assignments);
           boolean right = eval(t.right,assignments);
           return (!(left && right) && (left || right));
           /*
             The only exception: if it's a variable, we check its
             valuation via two steps: first, we figure out what index in
             the assignment string it refers to.  This is done by simply
             using the addVar function, since addVar returns the index the
             variable is/was placed in if it did not already exist.  It
             then checks to see if the character at that index is a '1'.
           */
       default:
           return (assignments.charAt(t.addVar(c)) == '1');
       }

   }

}



/*
 Very simple wrapper class.  This class just contains a Tree and a String
*/

class TreeString {
   Tree tree;
   String str;

   public TreeString(Tree t, String s) {
       tree =t;
       str = s;
   }

}

/* An instance of the Tree class represents our parse tree, and it holds
  everything important.  In this case, that consists of the value of the
  node of the parse tree, a pointer to the left and right children, and the
  array of characters telling us what variables have been seen so far in our
  parse tree
*/

class Tree {
   Tree left, right;
   char value;
   char[] vars;

   // Constructors.  We initialize the variables to the '!' character as our
   // sentinel.
   public Tree(Tree l, Tree r, char v) {
       left = l;
       right = r;
       value = v;
       vars = new char[10];
       for (int i = 0; i < 10; i++) {
           vars[i] = '!';
       }
   }
   public Tree(Tree l, Tree r, char v, char[] vs) {
       left = l;
       right = r;
       value = v;
       vars = vs;
   }

   /*
      When we're told to add a variable, we move through the array until we
      either find the variable, in which case we return its index.  If we
      find a !, we replace it with the variable we're adding and return
      that index.

      If the entire array is full, we return -1.
   */

   public int addVar(char c) {
       for (int i = 0; i <10; i++) {
           if (vars[i] == c)
               return i;
           if (vars[i] == '!') {
               vars[i] = c;
               return i;
           }
       }
       return -1;
   }

   /* This is only here for debugging purposes, but it's a nice general
      thing to know about Java.  You can override a lot of the built-in
      methods of objects you define so they act how you want.  In this
      case, overriding toString() defines what happens every time you call
      System.out.println on a tree.  Other common functions to override
      include equals and compareTo, but that's a lesson for another problem!
   */

   public String toString() {
       String l = "()";
       String r = "()";
       if (left != null)
           l = left.toString();
       if (right != null)
           r = right.toString();
       String varsi = "[";
       for (int i = 0; i <10; i++) {
           varsi += vars[i] + " ";
       }
       varsi += "]";
       return "("+ value + "," + left + "," + right + "," + varsi + ")" ;
   }

}
